@@ -253,7 +253,9 @@ class CacheOnDisk(CacheAbstract):
             except ImportError:
                 pass # no module _bsddb, ignoring exception now so it makes a ticket only if used
             except:
-                logger.error('corrupted file: %s' % self.shelve_name)
+                logger.error('corrupted file %s, deleting it!' \
+                                 % self.shelve_name)
+                os.unlink(self.shelve_name)
             if locker_locked:
                 portalocker.unlock(locker)
             if locker:
@@ -413,6 +415,10 @@ class Cache(object):
             cache_model = self.ram
 
         def tmp(func):
-            return lambda: cache_model(key, func, time_expire)
+            def action():
+                return cache_model(key, func, time_expire)
+            action.__name___ = func.__name__
+            action.__doc__ = func.__doc__
+            return action
 
         return tmp
