@@ -12,24 +12,28 @@ Functions required to execute app components
 FOR INTERNAL USE ONLY
 """
 
+import re
+import fnmatch
 import os
 import copy
 import random
 from storage import Storage, List
 from template import parse_template
 from restricted import restricted, compile2
-from fileutils import listdir
+from fileutils import mktree, listdir
 from myregex import regex_expose
 from languages import translator
-from sql import BaseAdapter, SQLDB, SQLField, DAL, Field
+from dal import BaseAdapter, SQLDB, SQLField, DAL, Field
 from sqlhtml import SQLFORM, SQLTABLE
 from cache import Cache
+from globals import current
 import settings
 from cfs import getcfs
 import html
 import validators
 from http import HTTP, redirect
 import marshal
+import shutil
 import imp
 import logging
 logger = logging.getLogger("web2py")
@@ -89,21 +93,24 @@ class LoadFactory(object):
         self.environment = environment
     def __call__(self, c=None, f='index', args=[], vars={},
                  extension=None, target=None,ajax=False,ajax_trap=False,
-                 url=None):
+                 url=None,user_signature=False, content='loading...',**attr):
         import globals
         target = target or 'c'+str(random.random())[2:]
+        attr['_id']=target        
         request = self.environment['request']
         if '.' in f:
             f, extension = f.split('.',1)
         if url or ajax:
             url = url or html.URL(request.application, c, f, r=request,
-                                  args=args, vars=vars, extension=extension)
+                                  args=args, vars=vars, extension=extension,
+                                  user_signature=user_signature)
             script = html.SCRIPT('web2py_component("%s","%s")' % (url, target),
                                  _type="text/javascript")
-            return html.TAG[''](script, html.DIV('loading...', _id=target))
+            return html.TAG[''](script, html.DIV(content,**attr))
         else:
+            if not isinstance(args,(list,tuple)):
+                args = [args]
             c = c or request.controller
-            other_environment = copy.copy(self.environment)
             other_request = globals.Request()
             other_request.application = request.application
             other_request.controller = c
@@ -112,21 +119,21 @@ class LoadFactory(object):
             other_request.args = List(args)
             other_request.folder = request.folder
             other_request.env = request.env
-            if not ajax_trap:
-                other_request.vars = request.vars
-                other_request.get_vars = request.get_vars
-                other_request.post_vars = request.post_vars
-            else:
-                other_request.vars = vars
-            other_environment['request'] = other_request
+            other_request.vars = request.vars
+            other_request.get_vars = request.get_vars
+            other_request.post_vars = request.post_vars
             other_response = globals.Response()
-            other_environment['response'] = other_response
-            other_response._view_environment = other_environment
             other_request.env.http_web2py_component_location = \
                 request.env.path_info
             other_request.env.http_web2py_component_element = target
             other_response.view = '%s/%s.%s' % (c,f, other_request.extension)
+
+            other_environment = copy.copy(self.environment)
+            other_response._view_environment = other_environment
+            other_environment['request'] = other_request
+            other_environment['response'] = other_response
             page = run_controller_in(c, f, other_environment)
+
             if isinstance(page, dict):
                 other_response._vars = page
                 for key in page:
@@ -136,14 +143,15 @@ class LoadFactory(object):
             js = None
             if ajax_trap:
                 link = html.URL(request.application, c, f, r=request,
-                                args=args, vars=vars, extension=extension),
+                                args=args, vars=vars, extension=extension,
+                                user_signature=user_signature),
                 js = "web2py_trap_form('%s','%s');" % (link, target)
             # not sure about this
             # for (name,value) in other_response.headers:
             #     if name == 'web2py-component-command':
             #         js += value
             script = js and html.SCRIPT(js,_type="text/javascript") or ''
-            return html.TAG[''](html.DIV(html.XML(page),_id=target),script)
+            return html.TAG[''](html.DIV(html.XML(page),**attr),script)
 
 
 def local_import_aux(name, force=False, app='welcome'):
@@ -210,22 +218,22 @@ def build_environment(request, response, session):
     environment = {}
     for key in html.__all__:
         environment[key] = getattr(html, key)
-
-    # Overwrite the URL function with a proxy
-    # url function which contains this request.
-    environment['URL'] = html._gURL(request)
-
     for key in validators.__all__:
         environment[key] = getattr(validators, key)
     if not request.env:
         request.env = Storage()
-    environment['T'] = translator(request)
+
+    current.request = request
+    current.response = response
+    current.session = session
+    current.T = environment['T'] = translator(request)
+    current.cache = environment['cache'] = Cache(request)
+
     environment['HTTP'] = HTTP
     environment['redirect'] = redirect
     environment['request'] = request
     environment['response'] = response
     environment['session'] = session
-    environment['cache'] = Cache(request)
     environment['DAL'] = DAL
     environment['Field'] = Field
     environment['SQLDB'] = SQLDB        # for backward compatibility
@@ -290,8 +298,8 @@ def compile_models(folder):
         fp = open(os.path.join(path, file), 'r')
         data = fp.read()
         fp.close()
-        filename = os.path.join(folder, 'compiled', ('models/'
-                                 + file).replace('/', '_'))
+        filename = os.path.join(folder, 'compiled','models',file)
+        mktree(filename)
         fp = open(filename, 'w')
         fp.write(data)
         fp.close()
@@ -331,21 +339,31 @@ def run_models_in(environment):
     """
 
     folder = environment['request'].folder
-    path = os.path.join(folder, 'compiled')
-    if os.path.exists(path):
-        for model in listdir(path, '^models_.+\.pyc$', 0):
+    c = environment['request'].controller
+    f = environment['request'].function
+    cpath = os.path.join(folder, 'compiled')
+    if os.path.exists(cpath):
+        for model in listdir(cpath, '^models_\w+\.pyc$', 0):
             restricted(read_pyc(model), environment, layer=model)
+        path = os.path.join(cpath, 'models')
+        models = listdir(path, '^\w+\.pyc$',0,sort=False)
+        compiled=True
     else:
-        models = listdir(os.path.join(folder, 'models'), '^\w+\.py$',
-                         0)
-        for model in models:
-            layer = model
-            if is_gae:
-                code = getcfs(model, model,
-                              lambda: compile2(open(model, 'r').read(),layer))
-            else:
-                code = getcfs(model, model, None)
-            restricted(code, environment, layer)
+        path = os.path.join(folder, 'models')
+        models = listdir(path, '^\w+\.py$',0,sort=False)
+        compiled=False
+    paths = (path, os.path.join(path,c), os.path.join(path,c,f))
+    for model in models:
+        if not os.path.split(model)[0] in paths:
+            continue
+        elif compiled:
+            code = read_pyc(model)
+        elif is_gae:
+            code = getcfs(model, model,
+                          lambda: compile2(open(model, 'r').read(),model))
+        else:
+            code = getcfs(model, model, None)
+        restricted(code, environment, layer=model)
 
 
 def run_controller_in(controller, function, environment):
@@ -370,6 +388,13 @@ def run_controller_in(controller, function, environment):
                        web2py_error=badf)
         restricted(read_pyc(filename), environment, layer=filename)
     elif function == '_TEST':
+        # TESTING: adjust the path to include site packages
+        from settings import global_settings
+        from admin import abspath, add_path_first
+        paths = (global_settings.gluon_parent, abspath('site-packages', gluon=True),  abspath('gluon', gluon=True), '')
+        [add_path_first(path) for path in paths]
+        # TESTING END
+
         filename = os.path.join(folder, 'controllers/%s.py'
                                  % controller)
         if not os.path.exists(filename):
@@ -426,23 +451,28 @@ def run_view_in(environment):
     folder = request.folder
     path = os.path.join(folder, 'compiled')
     badv = 'invalid view (%s)' % response.view
+    patterns = response.generic_patterns or []
+    regex = re.compile('|'.join(fnmatch.translate(r) for r in patterns))
+    allow_generic = regex.search('%s/%s.%s' % (request.controller,
+                                               request.function,
+                                               request.extension))
     if not isinstance(response.view, str):
         ccode = parse_template(response.view, os.path.join(folder, 'views'),
                                context=environment)
         restricted(ccode, environment, 'file stream')
     elif os.path.exists(path):
         x = response.view.replace('/', '_')
+        files = ['views_%s.pyc' % x]
+        if allow_generic:
+            files.append('views_generic.%s.pyc' % request.extension)
+        # for backward compatibility
         if request.extension == 'html':
-            # for backward compatibility
-            files = [os.path.join(path, 'views_%s.pyc' % x),
-                     os.path.join(path, 'views_%s.pyc' % x[:-5]),
-                     os.path.join(path, 'views_generic.html.pyc'),
-                     os.path.join(path, 'views_generic.pyc')]
-        else:
-            files = [os.path.join(path, 'views_%s.pyc' % x),
-                     os.path.join(path, 'views_generic.%s.pyc'
-                                  % request.extension)]
-        for filename in files:
+            files.append('views_%s.pyc' % x[:-5])
+            if allow_generic:
+                files.append('views_generic.pyc')
+        # end backward compatibility code
+        for f in files:
+            filename = os.path.join(path,f)
             if os.path.exists(filename):
                 code = read_pyc(filename)
                 restricted(code, environment, layer=filename)
@@ -452,9 +482,9 @@ def run_view_in(environment):
                    web2py_error=badv)
     else:
         filename = os.path.join(folder, 'views', response.view)
-        if not os.path.exists(filename):
+        if not os.path.exists(filename) and allow_generic:
             response.view = 'generic.' + request.extension
-        filename = os.path.join(folder, 'views', response.view)
+            filename = os.path.join(folder, 'views', response.view)
         if not os.path.exists(filename):
             raise HTTP(404,
                        rewrite.thread.routes.error_message % badv,
@@ -476,14 +506,10 @@ def remove_compiled_application(folder):
     Deletes the folder `compiled` containing the compiled application.
     """
     try:
-        path = os.path.join(folder, 'compiled')
-        for file in listdir(path):
-            os.unlink(os.path.join(path, file))
-        os.rmdir(path)
+        shutil.rmtree(os.path.join(folder, 'compiled'))
         path = os.path.join(folder, 'controllers')
-        for file in os.listdir(path):
-            if file.endswith('.pyc'):
-                os.unlink(os.path.join(path, file))
+        for file in listdir(path,'.*\.pyc$',drop=False):
+            os.unlink(file)
     except OSError:
         pass
 
