@@ -15,11 +15,10 @@ Contains the classes for the global used variables:
 """
 
 from storage import Storage, List
-from compileapp import run_view_in
 from streamer import streamer, stream_file_or_304_or_206, DEFAULT_CHUNK_SIZE
 from xmlrpc import handler
 from contenttype import contenttype
-from html import xmlescape
+from html import xmlescape, TABLE, TR, PRE
 from http import HTTP
 from fileutils import up
 from serializers import json, custom_json
@@ -37,11 +36,13 @@ import Cookie
 import os
 import sys
 import traceback
+import threading
 
 regex_session_id = re.compile('^([\w\-]+/)?[\w\-\.]+$')
 
 __all__ = ['Request', 'Response', 'Session']
 
+current = threading.local()  # thread-local storage for request-scope globals
 
 class Request(Storage):
 
@@ -76,7 +77,7 @@ class Request(Storage):
         self.extension = None
         self.now = datetime.datetime.now()
         self.is_restful = False
-        
+
     def compute_uuid(self):
         self.uuid = '%s/%s.%s.%s' % (
             self.application,
@@ -121,6 +122,7 @@ class Response(Storage):
     def __init__(self):
         self.status = 200
         self.headers = Storage()
+        self.headers['X-Powered-By'] = 'web2py'
         self.body = cStringIO.StringIO()
         self.session_id = None
         self.cookies = Cookie.SimpleCookie()
@@ -142,6 +144,7 @@ class Response(Storage):
             self.body.write(xmlescape(data))
 
     def render(self, *a, **b):
+        from compileapp import run_view_in
         if len(a) > 2:
             raise SyntaxError, 'Response.render can be called with two arguments, at most'
         elif len(a) == 2:
@@ -159,7 +162,7 @@ class Response(Storage):
         if view:
             import cStringIO
             (obody, oview) = (self.body, self.view)
-            (self.body, self.view) = (cStringIO.StringIO(), view)
+            (self.body, self.view) = (cStringIO.StringIO(), view)            
             run_view_in(self._view_environment)
             page = self.body.getvalue()
             self.body.close()
@@ -265,6 +268,28 @@ class Response(Storage):
 
         return handler(request, self, methods)
 
+    def toolbar(self):
+        from html import DIV, SCRIPT, BEAUTIFY, TAG, URL
+        BUTTON = TAG.button
+        admin = URL("admin","default","design",
+                    args=current.request.application)
+        from gluon.dal import thread
+        dbstats = [TABLE(*[TR(PRE(row[0]),'%.2fms' % (row[1]*1000)) \
+                               for row in i.db._timings]) \
+                       for i in thread.instances]
+        return DIV(
+            BUTTON('design',_onclick="document.location='%s'" % admin),
+            BUTTON('request',_onclick="jQuery('#request').slideToggle()"),
+            DIV(BEAUTIFY(current.request),_class="hidden",_id="request"),
+            BUTTON('session',_onclick="jQuery('#session').slideToggle()"),
+            DIV(BEAUTIFY(current.session),_class="hidden",_id="session"),
+            BUTTON('response',_onclick="jQuery('#response').slideToggle()"),
+            DIV(BEAUTIFY(current.response),_class="hidden",_id="response"),
+            BUTTON('db stats',_onclick="jQuery('#db-stats').slideToggle()"),
+            DIV(BEAUTIFY(dbstats),_class="hidden",_id="db-stats"),
+            SCRIPT("jQuery('.hidden').hide()")
+            )
+
 class Session(Storage):
 
     """
@@ -484,3 +509,4 @@ class Session(Storage):
                 del response.session_file
             except:
                 pass
+            
