@@ -26,7 +26,7 @@ from contrib.markmin.markmin2html import render
 
 from storage import Storage
 from highlight import highlight
-from utils import web2py_uuid
+from utils import web2py_uuid, hmac_hash
 
 import hmac
 import hashlib
@@ -132,7 +132,8 @@ def URL(
     env=None,
     hmac_key=None,
     hash_vars=True,
-    _request=None,
+    salt=None,
+    user_signature=None,
     scheme=None,
     host=None,
     port=None,
@@ -181,7 +182,6 @@ def URL(
     :param hash_vars: which of the vars to include in our hmac signature
         True (default) - hash all vars, False - hash none of the vars,
         iterable - hash only the included vars ['key1','key2']
-    :param _request: used internally for URL rewrite
     :param scheme: URI scheme (True, 'http' or 'https', etc); forces absolute URL (optional)
     :param host: string to force absolute URL with host (True means http_host)
     :param port: optional port number (forces absolute URL)
@@ -193,8 +193,16 @@ def URL(
 
     args = args or []
     vars = vars or {}
+    application = None
+    controller = None
+    function = None
 
-    application = controller = function = None
+    if not r:
+        if a and not c and not f: (f,a,c)=(a,c,f)
+        elif a and c and not f: (c,f,a)=(a,c,f)
+        from globals import current
+        if hasattr(current,'request'):
+            r = current.request
     if r:
         application = r.application
         controller = r.controller
@@ -214,6 +222,8 @@ def URL(
         else:
             function = f
 
+    function2 = '%s.%s' % (function,extension or 'html')
+
     if not (application and controller and function):
         raise SyntaxError, 'not enough information to build the url'
 
@@ -231,11 +241,16 @@ def URL(
         for val in vals:
             list_vars.append((key, val))
 
+    if user_signature:
+        from globals import current
+        if current.session.auth:
+            hmac_key = current.session.auth.hmac_key
+
     if hmac_key:
         # generate an hmac signature of the vars & args so can later
         # verify the user hasn't messed with anything
 
-        h_args = '/%s/%s/%s%s' % (application, controller, function, other)
+        h_args = '/%s/%s/%s%s' % (application, controller, function2, other)
 
         # how many of the vars should we include in our hash?
         if hash_vars is True:       # include them all
@@ -250,23 +265,26 @@ def URL(
         # re-assembling the same way during hash authentication
         message = h_args + '?' + urllib.urlencode(sorted(h_vars))
 
-        sig = hmac.new(hmac_key,message, hashlib.sha1).hexdigest()
+        sig = hmac_hash(message,hmac_key,salt=salt)
         # add the signature into vars
         vars['_signature'] = sig
         list_vars.append(('_signature', sig))
 
-    if extension:
-        function += '.' + extension
     if vars:
         other += '?%s' % urllib.urlencode(list_vars)
     if anchor:
         other += '#' + urllib.quote(str(anchor))
+    if extension:
+        function += '.' + extension
 
     if regex_crlf.search(''.join([application, controller, function, other])):
         raise SyntaxError, 'CRLF Injection Detected'
-    return rewrite.url_out(r or _request, env, application, controller, function, args, other, scheme, host, port)
+    url = rewrite.url_out(r, env, application, controller, function,
+                          args, other, scheme, host, port)
+    return url
 
-def verifyURL(request, hmac_key, hash_vars=True):
+
+def verifyURL(request, hmac_key=None, hash_vars=True, salt=None, user_signature=None):
     """
     Verifies that a request's args & vars have not been tampered with by the user
 
@@ -274,8 +292,9 @@ def verifyURL(request, hmac_key, hash_vars=True):
     :param hmac_key: the key to authenticate with, must be the same one previously
                     used when calling URL()
     :param hash_vars: which vars to include in our hashing. (Optional)
-                    Only uses the 1st value currently (it's really a hack for the _gURL.verify lambda)
-                    True (or undefined) means all, False none, an iterable just the specified keys
+                    Only uses the 1st value currently
+                    True (or undefined) means all, False none,
+                    an iterable just the specified keys
 
     do not call directly. Use instead:
 
@@ -304,6 +323,15 @@ def verifyURL(request, hmac_key, hash_vars=True):
     if not request.get_vars.has_key('_signature'):
         return False # no signature in the request URL
 
+    # check if user_signature requires
+    if user_signature:
+        from globals import current
+        if not current.session:
+            return False
+        hmac_key = current.session.auth.hmac_key
+    if not hmac_key:
+        return False
+
     # get our sig from request.get_vars for later comparison
     original_sig = request.get_vars._signature
 
@@ -317,9 +345,11 @@ def verifyURL(request, hmac_key, hash_vars=True):
 
     # always include all of the args
     other = args and urllib.quote('/' + '/'.join([str(x) for x in args])) or ''
-    h_args = '/%s/%s/%s%s' % (request.application,
-                              request.controller,
-                              request.function, other)
+    h_args = '/%s/%s/%s.%s%s' % (request.application,
+                                 request.controller,
+                                 request.function,
+                                 request.extension,
+                                 other)
 
     # but only include those vars specified (allows more flexibility for use with
     # forms or ajax)
@@ -347,8 +377,9 @@ def verifyURL(request, hmac_key, hash_vars=True):
             return False
     # build the full message string with both args & vars
     message = h_args + '?' + urllib.urlencode(sorted(h_vars))
+
     # hash with the hmac_key provided
-    sig = hmac.new(str(hmac_key), message, hashlib.sha1).hexdigest()
+    sig = hmac_hash(message,str(hmac_key),salt=salt)
 
     # put _signature back in get_vars just in case a second call to URL.verify is performed
     # (otherwise it'll immediately return false)
@@ -358,35 +389,7 @@ def verifyURL(request, hmac_key, hash_vars=True):
     # (I.E. was the message the same as the one we originally signed)
     return original_sig == sig
 
-def _gURL(request):
-    """
-    A proxy function for URL which contains knowledge
-    of a given request object.
-
-    Usage is exactly like URL except you do not have
-    to specify r=request!
-    """
-    def _URL(*args, **kwargs):
-        # If they use URL as just passing along
-        # args, we don't want to overwrite it and
-        # cause issues.
-        if not kwargs.has_key('r') and len(args) < 3:
-            kwargs['r'] = request
-            if len(args) == 1 and not 'f' in kwargs:
-                kwargs['f'] = args[0]
-                args = []
-            if len(args) == 2 and not 'f' in kwargs and not 'c' in kwargs:
-                kwargs['c'], kwargs['f'] = args[0], args[1]
-                args = []
-        kwargs['_request'] = request
-        return URL(*args, **kwargs)
-    _URL.__doc__ = URL.__doc__
-
-    # probably ugly work-around for the lambda call. Want for hash_vars to be an optional argument
-    _URL.verify = lambda request, hmac_key, *hash_vars: verifyURL(request, hmac_key, *hash_vars)
-
-    return _URL
-
+URL.verify = verifyURL
 
 ON = True
 
@@ -431,11 +434,14 @@ class XML(XmlComponent):
             'code',
             'pre',
             'img/',
+            'h1','h2','h3','h4','h5','h6',
+            'table','tr','td','div',
             ],
         allowed_attributes = {
             'a': ['href', 'title'],
             'img': ['src', 'alt'],
-            'blockquote': ['type']
+            'blockquote': ['type'],
+            'td': ['colspan'],
             },
         ):
         """
@@ -1250,13 +1256,20 @@ class HR(DIV):
 
 
 class A(DIV):
+
     tag = 'a'
+
     def xml(self):
-        if self['cid']:
+        if self['callback']:
+            self['_onclick']="ajax('%s',[],'%s');return false;" % \
+                (self['callback'],self['target'] or '')
+            self['_href'] = self['_href'] or '#null'
+        elif self['cid']:
             self['_onclick']='web2py_component("%s","%s");return false;' % \
                 (self['_href'],self['cid'])
         return DIV.xml(self)
 
+
 class EM(DIV):
 
     tag = 'em'
@@ -1892,7 +1905,7 @@ class MENU(DIV):
                 li = LI(A(name, _href=link))
             else:
                 li = LI(A(name, _href='#',
-                          _onclick='javascript:void(0):return false;'))
+                          _onclick='javascript:void(0);return false;'))
             if len(item) > 3 and item[3]:
                 li['_class'] = self['li_class']
                 li.append(self.serialize(item[3], level+1))
