@@ -8,6 +8,7 @@ License: LGPLv3 (http://www.gnu.org/licenses/lgpl.html)
 """
 
 import hashlib
+import hmac
 import uuid
 import random
 import thread
@@ -15,16 +16,16 @@ import time
 
 def md5_hash(text):
     """ Generate a md5 hash with the given text """
-
     return hashlib.md5(text).hexdigest()
 
-
-def hash(text, digest_alg = 'md5'):
+def simple_hash(text, digest_alg = 'md5'):
     """
     Generates hash with the given text using the specified
     digest hashing algorithm
     """
-    if not isinstance(digest_alg,str):
+    if not digest_alg:
+        raise RuntimeError, "simple_hash with digest_alg=None"
+    elif not isinstance(digest_alg,str):
         h = digest_alg(text)
     else:
         h = hashlib.new(digest_alg)
@@ -53,6 +54,15 @@ def get_digest(value):
     else:
         raise ValueError("Invalid digest algorithm")
 
+def hmac_hash(value, key, digest_alg='md5', salt=None):
+    if ':' in key:
+        digest_alg, key = key.split(':')
+    digest_alg = get_digest(digest_alg)
+    d = hmac.new(key,value,digest_alg)
+    if salt:
+        d.update(str(salt))
+    return d.hexdigest()
+
 web2py_uuid_locker = thread.allocate_lock()
 node_id = uuid.getnode()
 milliseconds = int(time.time() * 1e3)
